package jp.co.kayo.android.localplayer.adapter;

import java.util.List;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.appwidget.ColorSet;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.util.SdCardAccessHelper;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.SearchItem;
import jp.co.kayo.android.localplayer.util.bean.SearchItem.ITEM_TYPE;
import android.content.Context;
import android.os.Build;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.RatingBar;
import android.widget.TextView;

public class SearchItemAdapter extends ArrayAdapter<SearchItem> {
    LayoutInflater inflater;
    boolean isSdCard;
    ViewCache mViewcache;

    public SearchItemAdapter(Context context, List<SearchItem> list, ViewCache cache) {
        super(context, R.layout.search_list_item, list);
        inflater = (LayoutInflater) context
                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        mViewcache = cache;
        isSdCard = ContentsUtils.isSDCard(context);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder holder = null;
        ;
        if (convertView == null) {
            holder = new ViewHolder();
            convertView = inflater.inflate(R.layout.search_list_item, null,
                    false);
            convertView.setTag(holder);

            holder.layoutGroup = convertView.findViewById(R.id.layoutGroup);
            holder.layoutItem = convertView.findViewById(R.id.layoutItem);
            holder.text0 = (TextView) convertView.findViewById(R.id.text0);
            holder.text1 = (TextView) convertView.findViewById(R.id.text1);
            holder.text2 = (TextView) convertView.findViewById(R.id.text2);
            holder.text3 = (TextView) convertView.findViewById(R.id.text3);
            holder.imgCache = (ImageView) convertView
                    .findViewById(R.id.imgCache);
            holder.ratingBar1 = (RatingBar)convertView.findViewById(R.id.ratingBar1);
            
            mViewcache.getColorset().setColor(ColorSet.KEY_DEFAULT_PRI_COLOR, holder.text0);
            mViewcache.getColorset().setColor(ColorSet.KEY_DEFAULT_PRI_COLOR, holder.text1);
            mViewcache.getColorset().setColor(ColorSet.KEY_DEFAULT_SEC_COLOR, holder.text2);
            mViewcache.getColorset().setColor(ColorSet.KEY_DEFAULT_SEC_COLOR, holder.text3);
            
        } else {
            holder = (ViewHolder) convertView.getTag();
        }

        SearchItem item = getItem(position);

        if (item.type == ITEM_TYPE.GROUP) {
            holder.layoutGroup.setVisibility(View.VISIBLE);
            holder.layoutItem.setVisibility(View.GONE);
            holder.text0.setText(item.title);
        } else {
            holder.layoutGroup.setVisibility(View.GONE);
            holder.layoutItem.setVisibility(View.VISIBLE);
            if (item.type == ITEM_TYPE.ALBUM) {
                holder.text2.setVisibility(View.VISIBLE);
                holder.text3.setVisibility(View.GONE);
                holder.imgCache.setVisibility(View.GONE);
                holder.text1.setText(item.album);
                holder.text2.setText(item.artist);
                holder.ratingBar1.setRating(item.rating);
            } else if (item.type == ITEM_TYPE.ARTIST) {
                holder.text2.setVisibility(View.GONE);
                holder.text3.setVisibility(View.GONE);
                holder.imgCache.setVisibility(View.GONE);
                holder.text1.setText(item.artist);
                holder.ratingBar1.setRating(item.rating);
            } else if (item.type == ITEM_TYPE.SONG) {
                holder.text2.setVisibility(View.VISIBLE);
                holder.text3.setVisibility(View.VISIBLE);
                holder.imgCache.setVisibility(View.VISIBLE);
                holder.text1.setText(item.title);
                holder.text2.setText(item.album);
                holder.text3.setText(item.artist);
                if (isSdCard != true
                        && SdCardAccessHelper.existCachFile(item.data)) {
                    holder.imgCache.setVisibility(View.VISIBLE);
                } else {
                    holder.imgCache.setVisibility(View.GONE);
                }
                holder.ratingBar1.setRating(item.rating);
            }
        }

        return convertView;
    }

    @Override
    public boolean isEnabled(int position) {
        SearchItem item = getItem(position);
        return item.type != ITEM_TYPE.GROUP;
    }

    class ViewHolder {
        View layoutGroup;
        View layoutItem;
        TextView text0;
        TextView text1;
        TextView text2;
        TextView text3;
        ImageView imgCache;
        RatingBar ratingBar1;
    }
}
