package jp.co.kayo.android.localplayer.util.bean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioArtist;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.SearchItem.ITEM_TYPE;

import android.content.Context;
import android.database.Cursor;
import android.provider.BaseColumns;
import android.support.v4.content.AsyncTaskLoader;

public class SearchItemLoader extends AsyncTaskLoader<List<SearchItem>> {
    private String mFilterText;
    private ArrayList<SearchItem> mList;
    private ViewCache mViewCache;

    public SearchItemLoader(Context context, ViewCache viewcache, String filter) {
        super(context);
        this.mViewCache = viewcache;
        mFilterText = filter;
    }

    @Override
    public List<SearchItem> loadInBackground() {
        return makeList();
    }

    private List<SearchItem> makeList() {
        if (mList == null) {
            mList = new ArrayList<SearchItem>();
        } else {
            mList.clear();
        }
        
        Map<String, SearchItem> albums = new HashMap<String, SearchItem>();
        Map<String, SearchItem> artists = new HashMap<String, SearchItem>();
        ArrayList<SearchItem> songs = new ArrayList<SearchItem>();
        makeListSongs(songs);
        makeListAlbums(albums);
        makeListArtist(artists);
        if (albums.size() > 0) {
            SearchItem grp = new SearchItem();
            grp.type = ITEM_TYPE.GROUP;
            grp.title = getContext().getString(R.string.lb_tab_albums_name)
                    .concat("(").concat(Integer.toString(albums.size()))
                    .concat(")");
            mList.add(grp);
            mList.addAll(albums.values());
        }
        if (artists.size() > 0) {
            SearchItem grp = new SearchItem();
            grp.type = ITEM_TYPE.GROUP;
            grp.title = getContext().getString(R.string.lb_tab_artist_name)
                    .concat("(").concat(Integer.toString(artists.size()))
                    .concat(")");
            mList.add(grp);
            mList.addAll(artists.values());
        }
        if (songs.size() > 0) {
            SearchItem grp = new SearchItem();
            grp.type = ITEM_TYPE.GROUP;
            grp.title = getContext().getString(R.string.lb_tab_media_name)
                    .concat("(").concat(Integer.toString(songs.size()))
                    .concat(")");
            mList.add(grp);
            mList.addAll(songs);
        }

        return mList;
    }

    void makeListSongs(ArrayList<SearchItem> songs) {
        StringBuilder where = new StringBuilder();
        ArrayList<String> whereArgs = new ArrayList<String>();
        if (mFilterText != null && mFilterText.length() > 0) {
            where.append(MediaConsts.AudioMedia.TITLE).append(" like ");
            where.append("'%' || ? || '%'");
            whereArgs.add(mFilterText);
        }

        Cursor cur = null;
        try {

            cur = getContext().getContentResolver().query(
                    MediaConsts.MEDIA_CONTENT_URI, null, where.toString(),
                    whereArgs.toArray(new String[whereArgs.size()]),
                    MediaConsts.AudioMedia.TITLE);

            if (cur != null && cur.moveToFirst()) {
                do {
                    String albumKey = cur.getString(cur
                            .getColumnIndex(AudioMedia.ALBUM_KEY));
                    String artistKey = cur.getString(cur
                            .getColumnIndex(AudioMedia.ARTIST_KEY));
                    String album = cur.getString(cur
                            .getColumnIndex(AudioMedia.ALBUM));
                    String artist = cur.getString(cur
                            .getColumnIndex(AudioMedia.ARTIST));
                    String title = cur.getString(cur
                            .getColumnIndex(AudioMedia.TITLE));
                    int col = cur.getColumnIndex(MediaConsts.AudioMedia.FAVORITE_POINT);

                    SearchItem item = new SearchItem();
                    item.id = cur.getLong(cur.getColumnIndex(AudioMedia._ID));
                    item.type = ITEM_TYPE.SONG;
                    item.albumKey = albumKey;
                    item.artistKey = artistKey;
                    item.album = album;
                    item.artist = artist;
                    item.title = title;
                    item.duration = cur.getLong(cur
                            .getColumnIndex(AudioMedia.DURATION));
                    item.data = cur.getString(cur
                            .getColumnIndex(AudioMedia.DATA));
                    if (col != -1) {
                        item.rating = cur.getInt(col);
                    } else {
                        FavoriteInfo inf = mViewCache.getFavorite(getContext(), item.id, TableConsts.FAVORITE_TYPE_SONG);
                        item.rating = inf.rating;
                    }
                    songs.add(item);
                } while (cur.moveToNext());
            }
        } finally {
            if (cur != null) {
                cur.close();
            }
        }
    }

    void makeListAlbums(Map<String, SearchItem> albums) {
        StringBuilder where = new StringBuilder();
        ArrayList<String> whereArgs = new ArrayList<String>();
        if (mFilterText != null && mFilterText.length() > 0) {
            where.append(MediaConsts.AudioAlbum.ALBUM).append(" like ");
            where.append("'%' || ? || '%'");
            whereArgs.add(mFilterText);
        }

        Cursor cur = null;
        try {
            cur = getContext().getContentResolver().query(
                    MediaConsts.ALBUM_CONTENT_URI,
                    null, where.toString(),
                    whereArgs.toArray(new String[whereArgs.size()]),
                    MediaConsts.AudioAlbum.ALBUM);

            if (cur != null && cur.moveToFirst()) {
                do {
                    String albumKey = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ALBUM_KEY));
                    String album = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ALBUM));
                    String artist = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ARTIST));
                    int col = cur.getColumnIndex(MediaConsts.AudioAlbum.FAVORITE_POINT);

                    SearchItem item = new SearchItem();
                    item.id = cur.getLong(cur.getColumnIndex(BaseColumns._ID));
                    item.type = ITEM_TYPE.ALBUM;
                    item.albumKey = albumKey;
                    item.album = album;
                    item.artist = artist;
                    if (col != -1) {
                        item.rating = cur.getInt(col);
                    } else {
                        FavoriteInfo inf = mViewCache.getFavorite(getContext(), item.id, TableConsts.FAVORITE_TYPE_ALBUM);
                        item.rating = inf.rating;
                    }
                    albums.put(item.albumKey, item);
                } while (cur.moveToNext());
            }
        } finally {
            if (cur != null) {
                cur.close();
            }
        }
    }

    void makeListArtist(Map<String, SearchItem> artists) {
        StringBuilder where = new StringBuilder();
        ArrayList<String> whereArgs = new ArrayList<String>();
        if (mFilterText != null && mFilterText.length() > 0) {
            where.append(MediaConsts.AudioMedia.ARTIST).append(" like ");
            where.append("'%' || ? || '%'");
            whereArgs.add(mFilterText);
        }

        Cursor cur = null;
        try {
            cur = getContext().getContentResolver().query(
                    MediaConsts.ARTIST_CONTENT_URI,
                    null, where.toString(),
                    whereArgs.toArray(new String[whereArgs.size()]),
                    MediaConsts.AudioArtist.ARTIST);

            if (cur != null && cur.moveToFirst()) {
                do {
                    String artistKey = cur.getString(cur
                            .getColumnIndex(AudioArtist.ARTIST_KEY));
                    String artist = cur.getString(cur
                            .getColumnIndex(AudioArtist.ARTIST));
                    int max = cur.getInt(cur
                            .getColumnIndex(AudioArtist.NUMBER_OF_TRACKS));
                    int col = cur.getColumnIndex(MediaConsts.AudioArtist.FAVORITE_POINT);

                    SearchItem item = new SearchItem();
                    item.id = cur.getLong(cur.getColumnIndex(BaseColumns._ID));
                    item.type = ITEM_TYPE.ARTIST;
                    item.artistKey = artistKey;
                    item.artist = artist;
                    item.number_of_tracks = max;
                    if (col != -1) {
                        item.rating = cur.getInt(col);
                    } else {
                        FavoriteInfo inf = mViewCache.getFavorite(getContext(), item.id, TableConsts.FAVORITE_TYPE_ARTIST);
                        item.rating = inf.rating;
                    }
                    artists.put(item.artistKey, item);

                } while (cur.moveToNext());
            }
        } finally {
            if (cur != null) {
                cur.close();
            }
        }
    }

    @Override
    protected void onStartLoading() {
        super.onStartLoading();
        if (mList == null) {
            forceLoad();
        } else {
            deliverResult(mList);
        }
    }

}
