package jp.co.kayo.android.localplayer.util.bean;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import jp.co.kayo.android.localplayer.TagEditActivity;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioArtist;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.service.StreamCacherServer;
import jp.co.kayo.android.localplayer.util.bean.EditItem.ITEM_TYPE;
import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.support.v4.content.AsyncTaskLoader;

public class SelectedItemLoader extends
        AsyncTaskLoader<List<EditItem>> {
    private int mType;
    private long[] mSelectedItems;
    private ArrayList<EditItem> mList;
    private boolean isSDCard;

    public SelectedItemLoader(Context context, int type, long[] items) {
        super(context);
        mType = type;
        mSelectedItems = items;
        isSDCard = ContentsUtils.isSDCard(context);
    }

    @Override
    public List<EditItem> loadInBackground() {
        return makeList();
    }

    private List<EditItem> makeList() {
        if (mList == null) {
            mList = new ArrayList<EditItem>();
        } else {
            mList.clear();
        }

        if (mType == TagEditActivity.ALBUM) {
            if (isSDCard) {
                makeListAlbumsToSongs(mSelectedItems, mList);
            } else {
                makeListAlbums(mSelectedItems, mList);
            }
        } else if (mType == TagEditActivity.ARTIST) {
            if (isSDCard) {
                makeListArtistToSongs(mSelectedItems, mList);
            } else {
                makeListArtist(mSelectedItems, mList);
            }
        } else if (mType == TagEditActivity.MEDIA) {
            makeListSongs(mSelectedItems, mList);
        }

        return mList;
    }

    void makeListSongs(long[] selectedItems, ArrayList<EditItem> songs) {
        for (long mediaId : selectedItems) {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        ContentUris.withAppendedId(
                                MediaConsts.MEDIA_CONTENT_URI, mediaId),
                        null, null, null, null);

                if (cur != null && cur.moveToFirst()) {
                    EditItem item = new EditItem();
                    item.id = mediaId;
                    item.type = ITEM_TYPE.SONG;
                    item.albumKey = cur.getString(cur
                            .getColumnIndex(AudioMedia.ALBUM_KEY));
                    item.artistKey = cur.getString(cur
                            .getColumnIndex(AudioMedia.ARTIST_KEY));
                    item.album = cur.getString(cur
                            .getColumnIndex(AudioMedia.ALBUM));
                    item.artist = cur.getString(cur
                            .getColumnIndex(AudioMedia.ARTIST));
                    item.title = cur.getString(cur
                            .getColumnIndex(AudioMedia.TITLE));
                    item.year = cur.getInt(cur
                            .getColumnIndex(AudioMedia.YEAR));
                    item.duration = cur.getLong(cur
                            .getColumnIndex(AudioMedia.DURATION));
                    item.data = isSDCard?
                            cur.getString(cur.getColumnIndex(AudioMedia.DATA)):
                            StreamCacherServer.getCacheFileName(getContext(), cur.getString(cur.getColumnIndex(AudioMedia.DATA)));
                    songs.add(item);
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    void makeListAlbumsToSongs(long[] selectedItems,
            ArrayList<EditItem> songs) {
        ArrayList<String> albumKeys = new ArrayList<String>();
        for (long albumId : selectedItems) {
            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ALBUM_CONTENT_URI, null,
                        AudioAlbum._ID + "= ?",
                        new String[] { Long.toString(albumId) }, null);

                if (cur != null && cur.moveToFirst()) {
                    String albumKey = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ALBUM_KEY));
                    albumKeys.add(albumKey);
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }

        for (String albumKey : albumKeys) {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.MEDIA_CONTENT_URI, null,
                        AudioMedia.ALBUM_KEY + " = ?",
                        new String[] { albumKey },
                        AudioMedia.TRACK + "," + AudioMedia.DATA);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        EditItem item = new EditItem();
                        item.id = cur.getLong(cur
                                .getColumnIndex(AudioMedia._ID));
                        item.type = ITEM_TYPE.SONG;
                        item.albumKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM_KEY));
                        item.artistKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST_KEY));
                        item.album = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM));
                        item.artist = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST));
                        item.title = cur.getString(cur
                                .getColumnIndex(AudioMedia.TITLE));
                        item.year = cur.getInt(cur
                                .getColumnIndex(AudioMedia.YEAR));
                        item.duration = cur.getLong(cur
                                .getColumnIndex(AudioMedia.DURATION));
                        item.data = isSDCard?
                                cur.getString(cur.getColumnIndex(AudioMedia.DATA)):
                                StreamCacherServer.getCacheFileName(getContext(), cur.getString(cur.getColumnIndex(AudioMedia.DATA)));
                        songs.add(item);
                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    void makeListAlbums(long[] selectedItems, ArrayList<EditItem> albums) {
        for (long albumId : selectedItems) {
            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ALBUM_CONTENT_URI, null,
                        AudioAlbum._ID + "= ?",
                        new String[] { Long.toString(albumId) }, null);

                if (cur != null && cur.moveToFirst()) {
                    EditItem item = new EditItem();
                    item.id = albumId;
                    item.type = ITEM_TYPE.ALBUM;
                    item.albumKey = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ALBUM_KEY));
                    item.album = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ALBUM));
                    item.artist = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ARTIST));
                    item.firstyear = cur.getInt(cur
                            .getColumnIndex(AudioAlbum.FIRST_YEAR));
                    item.lastyear = cur.getInt(cur
                            .getColumnIndex(AudioAlbum.LAST_YEAR));
                    item.art = cur.getString(cur
                            .getColumnIndex(AudioAlbum.ALBUM_ART));
                    albums.add(item);
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    void makeListArtistToSongs(long[] selectedItems,
            ArrayList<EditItem> songs) {
        Set<String> artistNames = new HashSet<String>();
        for (long artistId : selectedItems) {
            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ARTIST_CONTENT_URI, null,
                        AudioAlbum._ID + "= ?",
                        new String[] { Long.toString(artistId) }, null);

                if (cur != null && cur.moveToFirst()) {
                    String artist = cur.getString(cur
                            .getColumnIndex(AudioArtist.ARTIST));
                    artistNames.add(artist);
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }

        for (String artist : artistNames) {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.MEDIA_CONTENT_URI, null,
                        AudioMedia.ARTIST + " = ?",
                        new String[] { artist }, null);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        EditItem item = new EditItem();
                        item.id = cur.getLong(cur
                                .getColumnIndex(AudioMedia._ID));
                        item.type = ITEM_TYPE.SONG;
                        item.albumKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM_KEY));
                        item.artistKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST_KEY));
                        item.album = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM));
                        item.artist = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST));
                        item.title = cur.getString(cur
                                .getColumnIndex(AudioMedia.TITLE));
                        item.year = cur.getInt(cur
                                .getColumnIndex(AudioMedia.YEAR));
                        item.duration = cur.getLong(cur
                                .getColumnIndex(AudioMedia.DURATION));
                        item.data = isSDCard?
                                cur.getString(cur.getColumnIndex(AudioMedia.DATA)):
                                StreamCacherServer.getCacheFileName(getContext(), cur.getString(cur.getColumnIndex(AudioMedia.DATA)));
                        songs.add(item);
                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    void makeListArtist(long[] selectedItems, ArrayList<EditItem> artists) {
        for (long artistId : selectedItems) {
            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ARTIST_CONTENT_URI, null,
                        AudioArtist._ID + "= ?",
                        new String[] { Long.toString(artistId) }, null);

                if (cur != null && cur.moveToFirst()) {
                    EditItem item = new EditItem();
                    item.id = artistId;
                    item.type = ITEM_TYPE.ARTIST;
                    item.artistKey = cur.getString(cur
                            .getColumnIndex(AudioArtist.ARTIST_KEY));
                    item.artist = cur.getString(cur
                            .getColumnIndex(AudioArtist.ARTIST));
                    artists.add(item);
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    @Override
    protected void onStartLoading() {
        super.onStartLoading();
        if (mList == null) {
            forceLoad();
        } else {
            deliverResult(mList);
        }
    }

}